"use strict";
// Copyright 2020 Google Inc. Use of this source code is governed by an
// MIT-style license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.
Object.defineProperty(exports, "__esModule", { value: true });
const extractZip = require("extract-zip");
const fs_1 = require("fs");
const node_fetch_1 = require("node-fetch");
const p = require("path");
const semver_1 = require("semver");
const shell = require("shelljs");
const tar_1 = require("tar");
shell.config.fatal = true;
// The current platform's operating system. Throws if the operating system
// is not supported by Dart Sass Embedded.
const OS = (() => {
    switch (process.platform) {
        case 'linux':
            return 'linux';
        case 'darwin':
            return 'macos';
        case 'win32':
            return 'windows';
        default:
            throw Error(`Platform ${process.platform} is not supported.`);
    }
})();
// The current platform's architecture. Throws if the architecture is not
// supported by Dart Sass Embedded.
const ARCH = (() => {
    switch (process.arch) {
        case 'ia32':
            return 'ia32';
        case 'x86':
            return 'ia32';
        case 'x64':
            return 'x64';
        default:
            throw Error(`Architecure ${process.arch} is not supported.`);
    }
})();
// The current platform's file extension for archives.
const ARCHIVE_EXTENSION = OS === 'windows' ? '.zip' : '.tar.gz';
// Directory that holds source files.
const BUILD_PATH = 'build';
/**
 * Gets the latest version of the Embedded Protocol. Throws if an error occurs.
 *
 * @param version - The Git ref to check out and build. Defaults to `master`.
 * @param path - Build from this path instead of pulling from Github.
 * @param release - Download the latest release instead of building from source.
 */
async function getEmbeddedProtocol(options) {
    var _a;
    const repo = 'embedded-protocol';
    if (options.release) {
        const latestRelease = await getLatestReleaseInfo({
            repo,
            tag: true,
        });
        await downloadRelease({
            repo,
            assetUrl: `https://github.com/sass/${repo}/archive/` +
                `${latestRelease.name.replace(' ', '-')}` +
                ARCHIVE_EXTENSION,
            outPath: BUILD_PATH,
        });
        fs_1.promises.rename(p.join(BUILD_PATH, `${repo}-${latestRelease.name.replace(' ', '-')}`), p.join(BUILD_PATH, repo));
    }
    else if (!options.path) {
        fetchRepo({
            repo,
            outPath: BUILD_PATH,
            ref: options.version,
        });
    }
    const repoPath = (_a = options.path) !== null && _a !== void 0 ? _a : p.join(BUILD_PATH, repo);
    buildEmbeddedProtocol(repoPath);
    await linkBuiltFiles(repoPath, p.join(options.outPath, repo));
}
exports.getEmbeddedProtocol = getEmbeddedProtocol;
/**
 * Gets the latest version of the Dart Sass wrapper for the Embedded Compiler.
 * Throws if an error occurs.
 *
 * @param version - If `release` is true, the version of the released binary to
 *   download (defaults to the latest version). If it's false, the Git ref to
 *   check out and build (defaults to master).
 * @param path - Build from this path instead of pulling from Github.
 * @param release - Download the latest release instead of building from source.
 */
async function getDartSassEmbedded(options) {
    const repo = 'dart-sass-embedded';
    if (options.release) {
        const release = options.version
            ? { tag_name: options.version, name: `sass_embedded ${options.version}` }
            : await getLatestReleaseInfo({
                repo,
            });
        await downloadRelease({
            repo,
            assetUrl: `https://github.com/sass/${repo}/releases/download/` +
                `${release.tag_name}/` +
                `${release.name.replace(' ', '-')}-` +
                `${OS}-${ARCH}` +
                ARCHIVE_EXTENSION,
            outPath: options.outPath,
        });
        fs_1.promises.rename(p.join(options.outPath, 'sass_embedded'), p.join(options.outPath, repo));
    }
    else if (options.path) {
        buildDartSassEmbedded(options.path);
        await linkBuiltFiles(p.join(options.path, 'build'), p.join(options.outPath, repo));
    }
    else {
        fetchRepo({
            repo,
            outPath: BUILD_PATH,
            ref: options.version,
        });
        buildDartSassEmbedded(p.join(BUILD_PATH, repo));
        await linkBuiltFiles(p.join(BUILD_PATH, repo, 'build'), p.join(options.outPath, repo));
    }
}
exports.getDartSassEmbedded = getDartSassEmbedded;
// Gets the ReleaseInfo of the latest release for `repo`. If `version` is given,
// throws an error if the latest version is not semver-compatible with
// `version`. If `tag` is true, gets the latest tag instead of release.
async function getLatestReleaseInfo(options) {
    console.log(`Getting version info for ${options.repo}.`);
    const response = await node_fetch_1.default('https://api.github.com/repos/sass/' +
        `${options.repo}/${options.tag ? 'tags' : 'releases'}`, {
        redirect: 'follow',
    });
    if (!response.ok) {
        throw Error(`Failed to get version info for ${options.repo}: ${response.statusText}`);
    }
    const latestRelease = JSON.parse(await response.text())[0];
    const latestVersion = options.tag
        ? latestRelease.name
        : latestRelease.tag_name;
    if (options.versionConstraint) {
        try {
            semver_1.satisfies(latestVersion, options.versionConstraint);
        }
        catch (_a) {
            throw Error(`Latest release ${latestVersion} is not compatible with ${options.versionConstraint}.`);
        }
    }
    console.log(`Latest release for ${options.repo} is ${latestVersion}.`);
    return latestRelease;
}
// Downloads the release for `repo` located at `assetUrl`, then unzips it into
// `outPath`.
async function downloadRelease(options) {
    console.log(`Downloading ${options.repo} release asset.`);
    const response = await node_fetch_1.default(options.assetUrl, {
        redirect: 'follow',
    });
    if (!response.ok) {
        throw Error(`Failed to download ${options.repo} release asset: ${response.statusText}`);
    }
    const releaseAsset = await response.buffer();
    console.log(`Unzipping ${options.repo} release asset to ${options.outPath}.`);
    await cleanDir(p.join(options.outPath, options.repo));
    const zippedAssetPath = `${options.outPath}/${options.repo}${ARCHIVE_EXTENSION}`;
    await fs_1.promises.writeFile(zippedAssetPath, releaseAsset);
    if (OS === 'windows') {
        await extractZip(zippedAssetPath, {
            dir: process.cwd(),
        });
    }
    else {
        tar_1.extract({
            file: zippedAssetPath,
            cwd: options.outPath,
            sync: true,
        });
    }
    await fs_1.promises.unlink(zippedAssetPath);
}
// Clones `repo` into `outPath`, then checks out the given Git `ref`.
function fetchRepo(options) {
    var _a;
    if (!fs_1.existsSync(p.join(options.outPath, options.repo))) {
        console.log(`Cloning ${options.repo} into ${options.outPath}.`);
        shell.exec(`git clone \
      --depth=1 \
      git://github.com/sass/${options.repo} \
      ${p.join(options.outPath, options.repo)}`, { silent: true });
    }
    const version = options.ref ? `commit ${options.ref}` : 'latest update';
    console.log(`Fetching ${version} for ${options.repo}.`);
    shell.exec(`git fetch --depth=1 origin ${(_a = options.ref) !== null && _a !== void 0 ? _a : 'master'}`, {
        silent: true,
        cwd: p.join(options.outPath, options.repo),
    });
    shell.exec('git reset --hard FETCH_HEAD', {
        silent: true,
        cwd: p.join(options.outPath, options.repo),
    });
}
// Builds the embedded proto at `repoPath` into a pbjs with TS declaration file.
function buildEmbeddedProtocol(repoPath) {
    const proto = p.join(repoPath, 'embedded_sass.proto');
    console.log(`Building pbjs and TS declaration file from ${proto}.`);
    const protocPath = OS === 'windows'
        ? '%CD%/node_modules/protoc/protoc/bin/protoc.exe'
        : 'node_modules/protoc/protoc/bin/protoc';
    const pluginPath = OS === 'windows'
        ? '%CD%/node_modules/.bin/protoc-gen-ts.cmd'
        : 'node_modules/.bin/protoc-gen-ts';
    shell.exec(`${protocPath} \
      --plugin="protoc-gen-ts=${pluginPath}" \
      --js_out="import_style=commonjs,binary:." \
      --ts_out="." \
      ${proto}`, { silent: true });
}
// Builds the Embedded Dart Sass executable from the source at `repoPath`.
function buildDartSassEmbedded(repoPath) {
    console.log('Downloading dart-sass-embedded dependencies.');
    shell.exec('dart pub upgrade', {
        cwd: repoPath,
        silent: true,
    });
    console.log('Building dart-sass-embedded executable.');
    shell.exec('dart run grinder protobuf pkg-standalone-dev', {
        cwd: repoPath,
        silent: true,
    });
}
// Links the built files at `builtPath` into `outPath`.
async function linkBuiltFiles(builtPath, outPath) {
    console.log(`Linking built files into ${outPath}.`);
    await cleanDir(outPath);
    if (OS === 'windows') {
        shell.cp('-R', builtPath, outPath);
    }
    else {
        // Symlinking doesn't play nice with Jasmine's test globbing on Windows.
        fs_1.promises.symlink(p.resolve(builtPath), outPath);
    }
}
// Ensures that `dir` does not exist, but its parent directory does.
async function cleanDir(dir) {
    await fs_1.promises.mkdir(p.dirname(dir), { recursive: true });
    try {
        await fs_1.promises.rmdir(dir, { recursive: true });
    }
    catch (_) {
        // If dir doesn't exist yet, that's fine.
    }
}
//# sourceMappingURL=utils.js.map